UNIT UMC_cl_DAO_Extract;
{$mode ObjFPC}{$H+}
INTERFACE
USES
    Classes
  , Sysutils
  , Controls
  , db
  , ZConnection
  , ZDataSet
  , Dialogs
//  , UMC_Types
//  , UMC_Const
//  , UMC_Const_DB
  , UMC_cl_EventHandler_OBJ
//  , UMC_cl_Object
  ;

TYPE
  TDBOpCode = ( opCode_Equal );


TYPE
  TA_NotifyEVT = ARRAY OF TNotifyEvent;

TYPE
  TCL_DAO_Core = CLASS( TObject )
    PRIVATE { Attribute }
      mv_TableName           : String;


    PRIVATE { Objecte }
      mo_EVTList        : TStringList;
      mo_Connection     : TZConnection;
      mo_DataSource     : TDataSource;
      mo_Query          : TZQuery;
      mo_Table          : TZTable;

    PRIVATE { Event-Handler }
      fon_Open          : TNotifyEvent;
      fon_Close         : TNotifyEvent;

    PRIVATE { SET'ter}
      PROCEDURE   ADD_EVT_On_Open( io_Value : TNotifyEvent );
      PROCEDURE   ADD_EVT_On_Close( io_Value : TNotifyEvent );
      PROCEDURE   CALL_Events( iv_EVTType : TEVT_Type; io_Field : TField );

    PRIVATE { GET'ter}
      FUNCTION    sGET_TableName : String;
      FUNCTION    oGET_DataSet   : TDataSet;
      FUNCTION    IS_Active              : Boolean;

    PROTECTED { Functions }
      FUNCTION    IS_Table : Boolean;

    PROTECTED { Procedures }
      PROCEDURE   INIT_Self;
      PROCEDURE   INIT_Handler;
      PROCEDURE   ASSIGN_Handler;
      PROCEDURE   SET_TableName( iv_Value : String );
    PROTECTED { Event-Handler }

    PROTECTED { Property's }
      PROPERTY    EVTList : TStringList
                  READ  mo_EVTList;
    PUBLIC
      CONSTRUCTOR Create( io_DSrc : TDataSource; io_DSet : TDataSet; iv_TabName : String = '' );
      DESTRUCTOR  Destroy;                                                                          OVERRIDE;

    PUBLIC { Functions }
    PUBLIC { Procedures }
      PROCEDURE   Open;
      PROCEDURE   Close;

    PUBLIC { Event-Handler }
    PUBLIC { Property's }
      PROPERTY    TableName : String
                  READ  sGET_TableName
                  WRITE SET_TableName;

      PROPERTY    DataSet : TDataSet
                  READ  oGET_DataSet;

      PROPERTY    DataSource : TDataSource
                  READ  mo_DataSource;

      PROPERTY    Query : TZQuery
                  READ  mo_Query;

      PROPERTY    Table : TZTable
                  READ  mo_Table;

      PROPERTY    ON_Open  : TNotifyEvent
                  READ  fon_Open
                  WRITE ADD_EVT_On_Open;

      PROPERTY    ON_Close : TNotifyEvent
                  READ  fon_Close
                  WRITE ADD_EVT_On_Close;
  END;



IMPLEMENTATION

CONST
  _CLS_Name_Query = 'TZQuery';
  _CLS_Name_Table = 'TZTable';




{===================================================================================================
===================================================================================================}
CONSTRUCTOR TCL_DAO_Core.Create( io_DSrc : TDataSource; io_DSet : TDataSet; iv_TabName : String = '' );
VAR lv_s : String;
BEGIN
  INHERITED Create;
  init_Self;

  mo_DataSource := io_DSrc;
  IF io_DSet.ClassName = _CLS_Name_Query THEN BEGIN
    mo_Query              := io_DSet AS TZQuery;
    mv_TableName          := iv_TabName;
    IF NOT Assigned( mo_DataSource.DataSet ) THEN
      mo_DataSource.DataSet := mo_Query;
    lv_s := mo_Query.Name;
  END;

  IF io_DSet.ClassName = _CLS_Name_Table THEN BEGIN
    mo_Table              := io_DSet AS TZTable;
    mv_TableName          := mo_Table.TableName;
    IF NOT Assigned( mo_DataSource.DataSet ) THEN
      mo_DataSource.DataSet := mo_Table;
    lv_s := mo_Table.Name;
  END;

  ASSIGN_Handler;
END;


{===================================================================================================
===================================================================================================}
DESTRUCTOR TCL_DAO_Core.Destroy;
BEGIN
  INHERITED Destroy;
END;


{###################################################################################################
####################################################################################################
                                       INITIALISIERUNG
####################################################################################################
###################################################################################################}

{===================================================================================================
===================================================================================================}
PROCEDURE TCL_DAO_Core.ADD_EVT_On_Open( io_Value : TNotifyEvent );
VAR lo_EVTOBJ : TCL_EventHandler_OBJ;
BEGIN
  IF NOT Assigned( mo_EVTList ) THEN
    mo_EVTList := TStringList.Create;

  lo_EVTOBJ := TCL_EventHandler_OBJ.Create( evt_Open, io_Value );
  mo_EVTList.AddObject( lo_EVTOBJ.Name, lo_EVTOBJ );
END;


{===================================================================================================
===================================================================================================}
PROCEDURE TCL_DAO_Core.ADD_EVT_On_Close( io_Value : TNotifyEvent );
VAR lo_EVTOBJ : TCL_EventHandler_OBJ;
BEGIN
  IF NOT Assigned( mo_EVTList ) THEN
    mo_EVTList := TStringList.Create;

  lo_EVTOBJ := TCL_EventHandler_OBJ.Create( evt_Close, io_Value );
  mo_EVTList.AddObject( lo_EVTOBJ.Name, lo_EVTOBJ );
END;



{===================================================================================================
===================================================================================================}
PROCEDURE TCL_DAO_Core.INIT_Self;
BEGIN
  mv_TableName           := '';
  mo_Connection          := NIL;
  mo_DataSource          := NIL;
  mo_Query               := NIL;
  mo_Table               := NIL;
  mo_EVTList             := NIL;
END;


{===================================================================================================
===================================================================================================}
PROCEDURE TCL_DAO_Core.INIT_Handler;
BEGIN
  fon_Open          := NIL;
  fon_Close         := NIL;
END;


{===================================================================================================
===================================================================================================}
PROCEDURE TCL_DAO_Core.ASSIGN_Handler;
BEGIN
END;



{###################################################################################################
####################################################################################################
                                       EVENT - Handler
####################################################################################################
###################################################################################################}

{===================================================================================================
===================================================================================================}
PROCEDURE TCL_DAO_Core.CALL_Events( iv_EVTType : TEVT_Type; io_Field : TField );
VAR lv_i : Integer; lo_Evt : TCL_EventHandler_OBJ;
BEGIN
  IF NOT Assigned( EVTList ) THEN EXIT;

  FOR lv_i := 0 TO EVTList.Count - 1 DO BEGIN
    lo_Evt := EVTList.Objects[ lv_i ] AS TCL_EventHandler_OBJ;
    IF lo_Evt.EVT_Type <> iv_EVTType THEN Continue;

    lo_Evt.CALL_Handler( Self, NIL );
  END;
END;




{###################################################################################################
####################################################################################################
                                       PROZEDUREN
####################################################################################################
###################################################################################################}


{===================================================================================================
===================================================================================================}
PROCEDURE TCL_DAO_Core.SET_TableName( iv_Value : String );
BEGIN
  mv_TableName := iv_Value;
  IF Assigned( Table ) THEN
    Table.TableName := iv_Value;
END;


{===================================================================================================
lv_p := mo_LST_EVT_Open.Items[ lv_i ];
addr( lo_evt ) := lv_p;
TNotifyEvent( lv_p )( Self );
===================================================================================================}
PROCEDURE TCL_DAO_Core.Open;
BEGIN
  DataSet.Open;
  CALL_Events( evt_Open, NIL );
END;


{===================================================================================================
===================================================================================================}
PROCEDURE TCL_DAO_Core.Close;
BEGIN
  DataSet.Close;
  CALL_Events( evt_Close, NIL );
END;



{###################################################################################################
####################################################################################################
                                       FUNKTIONEN
####################################################################################################
###################################################################################################}


{===================================================================================================
===================================================================================================}
FUNCTION TCL_DAO_Core.IS_Active : Boolean;
BEGIN
  IF IS_Table THEN
    Result := Table.Active
  ELSE
    Result := Query.Active;
END;


{===================================================================================================
===================================================================================================}
FUNCTION TCL_DAO_Core.sGET_TableName : String;
BEGIN
  Result := mv_TableName;
END;


{===================================================================================================
===================================================================================================}
FUNCTION TCL_DAO_Core.oGET_DataSet : TDataSet;
BEGIN
  Result := DataSource.DataSet;
END;


{===================================================================================================
===================================================================================================}
FUNCTION TCL_DAO_Core.IS_Table : Boolean;
BEGIN
  Result := Assigned( Table );
END;









END.

